# -*- coding: utf_8 -*-
# @Create   : 2021/5/11 11:14 上午
# @Author   : yh
# @Remark   : 在此定义所有的tree数据库操作方式

#################### OpenKey flag###############################
TRDB_OPKF_CREATEMAINKEY = 0x00010000
TRDB_OPKF_DELETEMAINKEY = 0x00020000
TRDB_OPKF_RENAME = 0x00040000
TRDB_OPKF_OPENEXIST = 0x00080000

TRDB_OPKF_READONLY = 0x00000001
TRDB_OPKF_SYSTEMKEY = 0x00000002
TRDB_OPKF_HIDE = 0x00000004
TRDB_OPKF_SEARCHKEY_PARENT = 0x00000008
TDDB_OPKF_CREATEDYNKEY = 0x00000100

TRDB_OPKF_OPENCHILD = 0x00000010
TRDB_OPKF_DOTPATH = 0x00000080
#####################################################################

BS_TRDB_DOTSEPARATOR = "."
BS_TRDB_NODESEPARATOR = "\\"

BS_TRDB_MAXFILENAME = 30
BS_TRDB_MAXSUBKEYPATH = 2048

BS_TRDB_MAXMAINKYELEN = 38
BS_TRDB_MAXNODENAELEN = 800
BS_TRDB_MAXRECORDNAMELEN = 800

BS_TRDB_MAXPWD = 30

#######################Insert key#################################

# TreeDB数据库的一些标志位
# 指示使用双向链式子键
BS_TRDB_FLAG_LINKNODE = 0x00000001
# 指示使用索引子键
BS_TRDB_FLAG_INDEXNODE = 0x00000002
# 指未使用双向链式属性
BS_TRDB_FLAG_LINKRECORD = 0x00000004
# 指示使用索引式属性
BS_TRDB_FLAG_INDEXRECORD = 0x00000008
# 指示使用双向链式主键
BS_TRDB_FLAG_LINKMAINNODE = 0x00000020
# 指示使用索引式主键
BS_TRDB_FLAG_INDEXMAINNODE = 0x00000040
######################################################################


###################################键索引类型#############################
# 设置此标志指示此列所建的索引为降序，否则为升序
BS_RITF_INDEX_DESC = 0x0020
"""
c++
struct bs_recordindextype
{
	i8 name[BS_TRDB_MAXRECORDNAMELEN];
	i8 uType;
	//设置此标志指示此列所建的索引为降序，否则为升序
#define BS_RITF_INDEX_DESC			0x0020
	ui16 uFlag;

};
python
bs_recordindextypes = []
bs_recordindextype = {"name": "age", "uType": VDT_I32, "uFlag": BS_RITF_INDEX_DESC}
bs_recordindextypes.append(bs_recordindextype)

"""

BS_RDT_LEN = (3 + BS_TRDB_MAXRECORDNAMELEN)

#######################################################
"""
c++
struct bs_query_item
{
	i8 name[BS_TRDB_MAXRECORDNAMELEN];
	i8 nCondition;
	chen::VariableData vLiData;
	chen::VariableData vEnd;

};
python
bs_query_items = []
bs_query_item = {"name": "_port", "nCondition": TRDB_FMC_EQUAL, "vLiData": "161", "vEnd": ""}
bs_query_items.append(bs_query_item)
"""

###########################bs_query_item:nCondition###############################

# 按属性查询时可用的查询条件
TRDB_FMC_GREATERTHAN = 0x01
TRDB_FMC_LESSTHAN = 0x02
TRDB_FMC_EQUAL = 0x03
TRDB_FMC_EQUALORGREATERTHAN = 0x04
TRDB_FMC_EQUALORLESSTHAN = 0x05
TRDB_FMC_UNEQUAL = 0x06
# 支持字符串通配符查询
TRDB_FMC_LIKE = 0x07
# 支持通配符查询不区分大小写
TRDB_FMC_NOCASE_LIKE = 0x08
TRDB_FMC_RANGE = 0x09

############################################################################

TRDB_QOTT_OR = 0x2
TRDB_QOTT_AND = 0x3

###########################Calculate type####################################

TRDB_CALCTYPE_COUNT = 0x01
TRDB_CALCTYPE_SUM = 0x02
TRDB_CALCTYPE_AVERAGE = 0x03
TRDB_CALCTYPE_MAX = 0x04
TRDB_CALCTYPE_MIN = 0x05

##############################################################################



# -*- coding: utf_8 -*-
# @Create   : 2021/5/27 14:08
# @Author   : yh
# @Remark   : 美信python框架

from .base import BaseMx
from .main import Mx
from .module import Module
from .view import View, Response
from .model import Model
from .server import Server
from .TreeDB import TreeDB
from .TableDB import TableDB
from .mqDB import MQ
from .DB import DB



# -*- coding: utf_8 -*-
# @Create   : 2021/5/11 11:14 上午
# @Author   : yh
# @Remark   : 在此定义所有的数据库错误

BS_NOERROR = 0
BS_FAILED = 1

# //////////////////common//////////////

BS_ERROR_UNKNOWN = 2
BS_ERROR_INVALIDPAGEPOOL = 3
BS_ERROR_FATALERROR = 4

BS_ERROR_AUTHFAILURE = 5
BS_ERROR_NAMELEN = 6
BS_ERROR_GETPAGE = 10
BS_ERROR_ADDDATATOPAGE = 11
BS_ERROR_INSERTBPTREE = 12
BS_ERROR_PAGEHEAD = 13
BS_ERROR_CREATEOBJECT = 14
BS_ERROR_GETFREEPAGE = 15
BS_ERROR_DATAF0RMAT = 16
BS_ERROR_BUFFERTOOSMALL = 17
BS_ERROR_COPYDATAFROMPAGE = 18
BS_ERROR_CHECKSUM = 19
BS_ERROR_OPENTREE = 20
BS_ERROR_DELETEBPTREE = 21
BS_ERROR_ALLOCMEMORY = 22
BS_ERROR_TIMEOUT = 23
BS_ERROR_INDEXTREE = 24
BS_ERROR_INDEXTREETYPE = 25
BS_ERROR_NONSUPPORT = 26
BS_ERROR_EMPTYBUFFER = 27
BS_ERROR_PARAMETER = 28
BS_ERROR_CLEARPAGEDATA = 29
BS_ERROR_DESTORYBPTREE = 30
BS_ERROR_PUSHPAGELIST = 31
BS_ERROR_POPPAGELIST = 32
BS_ERROR_PAGETOOSMALL = 33
BS_ERROR_OPENBPTREE = 34
BS_ERROR_BPTREEFIND = 35
BS_ERROR_BPTREEINSERT = 36
BS_ERROR_BPTREEDELETE = 37
BS_ERROR_BPTREEDESTORY = 38
BS_ERROR_BPTREEEDIT = 39
BS_ERROR_BPTREENOFIND = 40
BS_ERROR_BPTREEVISIT = 41
BS_ERROR_BPTREEISEMPTY = 42
BS_ERROR_OVERWRITETOPAGE = 43

# /////////////////////IT////////////////
BS_IT_ITEMEXISTALLREADY = 45

# ///////////////////IRST/////////////////
BS_IRST_PARSEKEYERROR = 50
BS_IRST_FINEDERROR = 51

# ////////////////////////////////////////////

BS_ERROR_STRLISTTOBUFFER = 55
BS_ERROR_BUFFERTOSTRLIST = 56
BS_ERROR_OPENTRANSACTION = 57
BS_ERROR_NOOPENTRANSACTION = 58
BS_ERROR_COMMITTRANSACTION = 59
BS_ERROR_ROLLBACKTRANSACTION = 60
BS_ERROR_TRANSACTIONALREADYBEGIN = 61
BS_ERROR_DATAPROTOCOL = 62
BS_ERROR_CONVERTDATA = 63
BS_ERROR_UNSERIALDATA = 64

# /////////////////bslist/////////////////

BS_LIST_EMPTY = 70
BS_LIST_INVALIDHEADPID = 71
BS_LIST_NOMOREITEM = 72
BS_LIST_BADITEMPAGEPOS = 73
BS_LIST_ITEMCOUNTERROR = 74
BS_LIST_PAGEDATATYPEERROR = 75

# /////////////////mq///////////////////

BS_MQ_EXISTALLREADY = 120
BS_MQ_NOEXIST = 121
BS_MQ_NOFINDPFHFLAG = 122
BS_MQ_MQHEADERROR = 123
BS_MQ_NAMEUNMATCHED = 124
BS_MQ_GETPOSTIONHEADERROR = 125
BS_MQ_MQNOINITIALIZATION = 126
BS_MQ_OPENINDEXTREE = 127
BS_MQ_INVALIDLEVEL = 128
BS_MQ_LOADLASTRECORDFAILED = 129
BS_MQ_ADDDATATOPAGEFAILED = BS_ERROR_ADDDATATOPAGE
BS_MQ_EMPTY = 131
BS_MQ_RELOADMQ = 132
BS_MQ_RELEASEMQHEAD = 134
BS_MQ_VISITMQNAMETREE = 135
BS_MQ_NOFINDMQ = 136
BS_MQ_ISNOTINDEXMQ = 137
BS_MQ_NOANYMQ = 138

# ///////////////treedb/////////////////
BS_TRDB_NOAUTHORITY_CREATEDBFILE = 157
BS_TRDB_DBFILEALREADYEXIST = 158
BS_TRDB_INITDBFILE = 159
BS_TRDB_NOINITIALIZATION = 160
BS_TRDB_MAINKEYISEMPTY = 161
BS_TRDB_MAINKEYALREADYEXIST = 162
BS_TRDB_READMAINKEYTREEROOTFAILED = 163
BS_TRDB_WRITEMAINKEYTREEROOTFAILED = 164
BS_TRDB_NOFINDPFHFLAG = 165
BS_TRDB_TRDBHEADERROR = 166
BS_TRDB_NODENAMELENERROR = 167
BS_TRDB_NODENAMEMISMATCH = 168
BS_TRDB_MAINKEYNOEXIST = 169
BS_TRDB_NOAUTHORITY_CREATESYSTEMKEY = 170
BS_TRDB_FINDMAINKEYEXCEPTION = 171
BS_TRDB_NOFINDMAINKEY = 172
BS_TRDB_NOFINDNODEPAGEFLAG = 173
BS_TRDB_OPENINDEXTREEFAILED = 174
BS_TRDB_SUBKEYALREADYEXIST = 175
BS_TRDB_NOFINDITEM = 176
BS_TRDB_UNKNOWNSUBKEYMODE = 177
BS_TRDB_INSERTINDEXTREEFAILED = BS_ERROR_INSERTBPTREE
BS_TRDB_NAMEISEMPTY = 179
BS_TRDB_PROPERTYALREADYEXIST = 180
BS_TRDB_PROPERTYNAMEALREADYEXIST = 181
BS_TRDB_DELETEINDEXTREEFAILED = 182
BS_TRDB_INDEXTREEROOTISNULL = 183
BS_TRDB_DELETEPROPERTY = 184
BS_TRDB_VISITNODEINDEXTREE = 185
BS_TRDB_FLAGUNKNOW = 186
BS_TRDB_PARSEDOTPATHFAILED = 187
BS_TRDB_PARSENODEPATHFAILED = 188
BS_TRDB_NOFINDTREEDBFILE = 189
BS_TRDB_CANNOTRENAMEMAINKEY = 190
BS_TRDB_CANNOTDELETEMAINKEY = 191
BS_TRDB_INDEXDATATYPEERROR = 200
BS_TRDB_NOSUPPORTINDEXTYPE = 201
BS_TRDB_NOFINDSEARCHKEYPARENTFLAG = 202
BS_TRDB_NOFINDINDEXROOTKEY = 203
BS_TRDB_CONFLICTWITHSYSTEMKEY = 204
BS_TRDB_NOFINDEXPRESSION = 205
BS_TRDB_DIFREENTDATATYPE = 206
BS_TRDB_CONDITIONERROR = 207
BS_TRDB_QUERYCONDITIONTYPEERROR = 208
BS_TRDB_NOFINDINDEXNODE = 209
BS_TRDB_NOFINDINDEXKEYFLAG = 211
BS_TRDB_NOFINDINDEXRNODERECORD = 212
BS_TRDB_NOFINDRECORDROOT = 213
BS_TRDB_NOFINDINDEXVALUENODEFLAG = 214
BS_TRDB_CALCULATEEXPRDATAERROR = 215
BS_TRDB_UNKNOWOPERATOR = 216
BS_TRDB_EXPRESSIONFORMAT = 217
BS_TRDB_EXPRESSIONDATACOUNTERROR = 218
BS_TRDB_NOSUPPORTDATATYPE = 219
BS_TRDB_CONVERTEXPRESSIONERROR = 220
BS_TRDB_UNKNOWCALCULATETYPE = 221
BS_TRDB_EXPRESSIONINDEXERROR = 222
BS_TRDB_ISNOTSEARCHKEYPARENT = 223

# ///////////////tabledb////////////////////////////////////////

BS_TBDB_NOINITIALIZATION = 260
BS_TBDB_USERALREADYEXIST = 261
BS_TBDB_INSERTINDEXTREEFAILED = BS_ERROR_INSERTBPTREE
BS_TBDB_DBALREADYEXIST = 263
BS_TBDB_NOFINDDB = 264
BS_TBDB_TABLEALREADYEXIST = 265
BS_TBDB_CREATEFIELDBUFFER = 266
BS_TBDB_UNKNOWNDATATYPE = 267
BS_TBDB_PRIMARYKEYWITHOUTUNIQUE = 268
BS_TBDB_PRIMARYREPEATASSIGN = 269
BS_TBDB_READFIELDHEAD = 270
BS_TBDB_FIELDFLAGERROR = 271
BS_TBDB_FIELDPOSERROR = 272
BS_TBDB_READFIELDINFOERROR = 273
BS_TBDB_FIELDTYPEERROR = 274
BS_TBDB_UNKNOWNFIELDTYPE = 275
BS_TBDB_DATALENGTHERROR = 276
BS_TBDB_WRITEFIELDINFOERROR = 277
BS_TBDB_CREATEINDEXTYPEERROR = 278
BS_TBDB_DYNPAGEERROR = 279
BS_TBDB_NOFINDTABLE = 280
BS_TBDB_GETTABLELOCKFAILED = 281
BS_TBDB_RECORDCBLENERROR = 282
BS_TBDB_RECORDPRIMARYKEYERROR = 283
BS_TBDB_SEEKPAGEPOSERROR = 284
BS_TBDB_INDEXERROR = 285
BS_TBDB_TABLECBLENERROR = 286
BS_TBDB_TABLEWASDELETED = 287
BS_TBDB_NOFINDRECORD = 288
BS_TBDB_CONDITIONERROR = 289
BS_TBDB_UNFIXEDFIELD = 290
BS_TBDB_RECORDSIZEERROR = 291
BS_TBDB_NOFINDFIELD = 292
BS_TBDB_FIELDCOUNTERROR = 293
BS_TBDB_STRLISTTOBUFFERERROR = BS_ERROR_STRLISTTOBUFFER
BS_TBDB_BUFFERTOSTRLISTERROR = BS_ERROR_BUFFERTOSTRLIST
BS_TBDB_NOFINDDYN = 296
BS_TBDB_NOTALERTINFO = 297
BS_TBDB_FIXEDRDSIZEERROR = 298
BS_TBDB_NOSUPPORTBINARYPRIMARYKEY = 299
BS_TBDB_NOSUPPORTBINARYINDEX = 300
BS_TBDB_FIELDSIZEERROR = 301
BS_TBDB_NOSUPPORTBINCONDITION = 302
BS_TBDB_DATABUFFERERROR = 303
BS_TBDB_CONTINUE = 304
BS_TBDB_END = 305

# ///////////////////////////////////////////////////////////////////////////

BSAPI_ERROR_PARAMETER = 1000
BSAPI_ERROR_CREATEOBJECT = 1001
BSAPI_ERROR_CONNECTSERVER = 1002
BSAPI_ERROR_ALLOCMEMORY = 1003
BSAPI_ERROR_NOSUPPORT_NETIOTYPE = 1004
BSAPI_ERROR_SENDDATA = 1005
BSAPI_ERROR_RECVDATA = 1006
BSAPI_ERROR_DATACHECK = 1007
BSAPI_ERROR_INVALIDHANDLE = 1008
BSAPI_ERROR_REOPENMQ = 1009
BSAPI_ERROR_SERIALDATA = 1010
BSAPI_ERROR_UNSERIALDATA = 1011
BSAPI_ERROR_DATAFORMAT = 1012
BSAPI_ERROR_CREATESTRINGLIST = 1013
BSAPI_ERROR_CREATESTRINGLISTBUFFER = 1014
BSAPI_ERROR_NOSUPPORT_DATATYPE = 1015
BSAPI_ERROR_FIELDDATA = 1016
BSAPI_ERROR_RECORDDATA = 1017
BSAPI_ERROR_PARSERECORDDATA = 1018
BSAPI_ERROR_ERRORHANDLE = 1019
BSAPI_ERROR_RECORDSET = 1020
BSAPI_CURSOR_END = 1021
BSAPI_CURSOR_BEGIN = 1022
BSAPI_ERROR_SESSIONCLOSED = 1023
BSAPI_ERROR_TRANHANDLEABORTED = 1024

error_dict = {
    40: '打开键失败',
    176: '找不到此属性'
}



# -*- coding: utf_8 -*-
# @Create   : 2021/6/25 16:59
# @Author   : yh
# @Remark   : mx框架Server层
from .conf_base import ConfBase


class Server(ConfBase):

    def __init__(self):
        super().__init__()
        self.db = getattr(self, '__db__')() if hasattr(self, '__db__') else None
        self.model = getattr(self, '__model__') if hasattr(self, '__model__') else None

    def list(self, *args, **kwargs):
        """
        展示数据的base方法
        """
        return self.db.list(*args, **kwargs)

    def create(self, *args, **kwargs):
        """
        添加数据的base方法
        """
        return self.db.create(**self.model(*args, **kwargs).dict())

    def update(self, code: str,  *args, **kwargs):
        """
        更新数据的base方法
        :param code: 要更新的键
        """
        return self.db.update(code, **self.model(*args, **kwargs).dict())

    def delete(self, code: str):
        """
        删除数据的base方法
        :param code: 要删除的键
        """
        return self.db.delete(code)



# -*- coding: utf_8 -*-
# @Create   : 2021/9/18 14:22
# @Author   : yh
# @Remark   : 存放消息队列的操作方法
from .BaseDB import BaseDB
from .db_def.def_mq import BSMQ_OF_CREATENEW, BSMQ_OF_OPENEXIST, BSMQ_OT_COMMONMQ, BS_TIMER_INFINITE
from .db_def.def_type import type_map
from .exception import DBError, DataError


class MQ(BaseDB):

    def open(self, name: str, pwd: str = '', host: str = None, port: int = None,
             flag: int = None, path_flag: bool = False):
        """
        打开消息队列

        :param name: 队列名称
        :param pwd: 密码
        :param host: 主机名
        :param port: 端口号
        :param flag: 创建的队列类型，默认普通队列，如果需要创建优先队列或其他类型请自行传值
        :param path_flag: 不存在的队列是否需要自动创建（默认不自动创建）
        :return: 类对象
        """
        host = host or self.host
        port = port or self.port

        if path_flag:
            open_flag = BSMQ_OF_OPENEXIST if name in self.mq_list(host, port) else BSMQ_OF_CREATENEW
        else:
            open_flag = BSMQ_OF_OPENEXIST
        flag = flag or BSMQ_OT_COMMONMQ

        try:
            self.exec_handle('bs_mq_reopen', name, pwd, open_flag, flag, host, port)
        except (DBError, DataError):
            try:
                self.exec('bs_mq_open', name, pwd, open_flag, flag, host, port)
            except DBError as e:
                raise DBError(e.err_code, '打开消息队列[%s]失败' % name)

        return self

    def mq_list(self, host: str = None, port: str = None) -> list:
        """
        获取所有mq名称列表
        """
        host = host or self.host
        port = port or self.port

        mq_list = list()
        self.exec_handle('bs_mq_query_all_names', mq_list, host, port)
        return mq_list

    def push(self, data: str, data_type: int = None, label: str = '', level: int = 1) -> None:
        """
        推送数据到消息队列
        
        :param data: 要推送的数据
        :param data_type: 数据的类型，不传会自动获取
        :param label: 数据标识
        :param level: 优先级（仅对优先队列有效）
        :return:
        """
        if data_type:
            pass
        elif type_map.get(type(data).__name__):
            data_type = type_map[type(data).__name__]
        else:
            raise DataError('无法获取到value的类型！如果您未手动传入类型或确信传入类型正确，请于type_map中添加此类型')

        self.exec_handle('bs_mq_push', data, data_type, label, level)

    def pop(self, time_out: int = 0, is_peek: bool = False):
        """
        从消息队列中取出一条数据
        :param time_out: 如果队列为空，此变量可设置等待时间，如果设置999999则认为无限等待
        :param is_peek: 此变量为假则把接收到的消息从队列中清除，为真只取回消息的拷贝
        :return: 取到的数据类
        """
        try:
            res = self.exec_handle('bs_mq_pop', BS_TIMER_INFINITE if time_out == 999999 else time_out, is_peek)
        except DBError as e:
            if e.err_code == 131:
                res = (None, None, None, None, None)
            else:
                raise DBError(e.err_code, e.msg)

        class Data:
            def __init__(self, data: tuple):
                self.data = data[1]
                self.label = data[2]
                self.time = data[3]

            def __str__(self):
                return str({'data': self.data, 'label': self.label, 'time': self.time})

        return Data(res)



# -*- coding: utf_8 -*-
# @Create   : 2021/5/17 9:46
# @Author   : yh
# @Remark   : session处理类
import typing as t
import json
from urllib.parse import unquote

from .conf_base import ConfBase
from .exception import CError, HTTPMethodError, DataError, AuthError
from .def_http_code import HttpCode, HttpType


class Request(ConfBase):
    """
    session处理基类
    """

    def __init__(self, session, *args, **kwargs):
        super().__init__()
        self.session = session  # session会话
        self.status_code = HttpCode.st_200_ok  # 默认的响应码
        self.response_content_type = 'application/json; charset=utf-8'  # 默认的response类型

        self._request_headers_cls = None
        self._response_headers_cls = None
        self._headers = None
        self._cookie = None
        self._url_cls = None
        self._peer_ip = None
        self._GET = None
        self._POST = None
        self._web_path = None
        self._request_type = None
        self._content_type = None
        self._session_id = None
        self._company = None
        self._user = None

    @property
    def session_id(self):
        """
        获取session id
        """
        if self._session_id:
            return self._session_id
        else:
            from mxsoft import CheckMxSession
            self._session_id = CheckMxSession(self.session)
            return self._session_id

    @property
    def company(self):
        """
        获取公司
        """
        if self._company:
            return self._company
        else:
            from bsmiddle import GetSessionCompany
            flag, company = GetSessionCompany(self.session_id)
            if flag:
                self._company = company
                return self._company
            else:
                raise AuthError('GetSessionCompany返回码异常：%s, 获取公司失败' % flag)

    @property
    def user(self):
        """
        获取用户
        """
        if self._user:
            return self._user
        else:
            from bsmiddle import GetSessionUserId
            flag, user = GetSessionUserId(self.session_id)
            if flag:
                self._user = user
                return self._user
            else:
                raise AuthError('GetSessionUserId返回码异常：%s, 获取用户失败' % flag)

    @property
    def request_headers_cls(self):
        """
        获取request_headers类
        """
        if self._request_headers_cls:
            return self._request_headers_cls
        else:
            try:
                self._request_headers_cls = self.session.GetHttpRequestHead()
            except TypeError:
                raise CError('session.GetHttpRequestHead()')
            return self._request_headers_cls

    @property
    def response_headers_cls(self):
        """
        获取response_headers类
        :return:
        """
        if self._response_headers_cls:
            return self._response_headers_cls
        else:
            try:
                self._response_headers_cls = self.session.GetHttpResponseHead()
            except TypeError:
                raise CError('session.GetHttpResponseHead()')
            return self._response_headers_cls

    @property
    def headers(self) -> dict:
        """
        获取并返回所有请求头
        """
        if self._headers:
            return self._headers
        else:
            header_list = list()
            header_dict = dict()
            try:
                self.request_headers_cls.GetAllHead(header_list)
            except TypeError:
                raise TypeError('session.GetHttpRequestHead().GetAllHead()')
            for header in header_list:
                header_class = header.split(': ')
                if len(header_class) == 2:
                    header_dict[header_class[0]] = header_class[1]
            self._headers = header_dict
            return self._headers

    @property
    def cookie(self) -> dict:
        """
        获取请求cookie
        """
        if self._cookie:
            return self._cookie
        else:
            cookie_dict = dict()
            try:
                cookie_str = self.request_headers_cls.GetCookie()
            except TypeError:
                raise CError('session.GetHttpRequestHead().GetCookie()')
            cookie_list = cookie_str.split('; ')

            for cookie in cookie_list:
                k, v = cookie.split('=')
                cookie_dict[k] = v
            self._cookie = cookie_dict
            return self._cookie

    @property
    def url_cls(self):
        """
        获取请求url类
        class GetUrl:
            GetFragment()、GetHost()、 GetPass()、GetPath()、GetQuery()、GetQueryStringMap()、GetQueryStringMapEx()、
            GetScheme()、GetUrlType()、GetUser()
        """
        if self._url_cls:
            return self._url_cls
        else:
            try:
                self._url_cls = self.session.GetUrl()
            except TypeError:
                raise CError('session.GetUrl()')
            return self._url_cls

    @property
    def url(self) -> str:
        """
        获取请求url
        """
        try:
            return self.url_cls.GetPath()
        except TypeError:
            raise CError('session.GetUrl().GetPath()')

    @property
    def peer_ip(self):
        """
        获取请求ip
        """
        if self._peer_ip:
            return self._peer_ip
        else:
            from mxsoft import GetPeerIP
            self._peer_ip = GetPeerIP(self.session)
            return self._peer_ip

    @property
    def callback(self) -> str:
        """
        返回回调标志
        """
        return self.GET.get('callbackparam') or ""

    @property
    def GET(self) -> dict:
        """
        获取get方法传递的参数
        """
        if self._GET:
            return self._GET
        else:
            query = dict()
            try:
                self.url_cls.GetQueryStringMapEx(query)
            except TypeError:
                raise CError('GetUrl().GetQueryStringMapEx()')
            self._GET = query
            return self._GET

    @property
    def POST(self) -> dict:
        """
        获取post方法传递的参数
        """
        if self._POST:
            return self._POST
        else:
            parse_dict = {
                'application/x-www-form-urlencoded': self._post_x_www_form_urlencoded,
                'application/json': self._post_json
            }
            try:
                post_length, post_data = self.session.GetPostString()
            except TypeError:
                raise CError('GetPostString()')
            try:
                data = parse_dict.get(
                    self.content_type.split(';')[0] if self.content_type else 'application/x-www-form-urlencoded')(
                    post_data)
            except TypeError:
                raise DataError('不支持的body参数类型: %s，目前支持的类型(%s)' % (self.content_type, ','.join(parse_dict.keys())))
            self._POST = data
            return self._POST

    def _post_form_data(self):
        """
        解析form-data
        """
        # TODO 未来再支持form_data

    @staticmethod
    def _post_x_www_form_urlencoded(post_data: str) -> dict:
        """
        解析x-www-form-urlencoded
        """
        return {unquote(post.split('=')[0]): unquote(post.split('=')[1]) if len(post.split('=')) == 2 else ''
                for post in post_data.split('&')}

    @staticmethod
    def _post_json(post_data: str) -> dict:
        """
        解析json
        """
        return json.loads(post_data)

    @property
    def web_path(self) -> str:
        """
        获取web服务所在目录
        """
        if self._web_path:
            return self._web_path
        else:
            try:
                self._web_path = self.session.GetRootPath()
            except TypeError:
                raise CError('session.GetRootPath()')
            return self._web_path

    @property
    def request_type(self) -> str:
        """
        获取请求类型
        """
        if self._request_type:
            return self._request_type
        else:
            try:
                res = self.request_headers_cls.GetRequestType()
            except TypeError:
                raise CError('session.GetHttpRequestHead().GetRequestType()')
            self._request_type = HttpType.type_dict[res] if HttpType.type_dict.get(res) else HttpType.type_dict[0]
            return self._request_type

    @property
    def content_type(self):
        """
        获取请求content_type
        """
        if self._content_type:
            return self._content_type
        else:
            self._content_type = self.headers.get('Content-Type') or self.headers.get('content-type')
            return self._content_type

    def add_header(self, header: str, value: str):
        """
        添加响应头
        :param header: 要添加的响应头
        :param value: 响应头的值
        """
        try:
            self.response_headers_cls.SetAddHead(str(header), str(value))
        except TypeError:
            raise CError('session.GetHttpResponseHead().SetAddHead()')


class Response:
    """
    响应类
    """

    def __init__(self, msg: t.Any, *args, **kwargs):
        self.msg = msg

    def __str__(self):
        return self.msg


class View(ConfBase):
    def __init__(self, request: "Request", *args, **kwargs):
        super().__init__()
        self.request = request

    def dispatch_request(self):
        """
        根据请求类型调用对应的类方法
        """

        meth = getattr(self, self.request.request_type.lower(), None)

        if meth is None and self.request.request_type == "HEAD":
            meth = getattr(self, "get", None)

        if not meth:
            raise HTTPMethodError(self.request.request_type)
        return self.request, meth()



# -*- coding: utf_8 -*-
# @Create   : 2021/5/27 14:17
# @Author   : yh
# @Remark   : 模块类
import typing as t

from .base import BaseMx


class Module(BaseMx):

    def __init__(self, name, url_prefix: str = None, **options: t.Any) -> None:
        super().__init__()
        self.name = name
        self.url_prefix = url_prefix

    def add_url_rule(self, url: str, func: t.Callable, **options: t.Any) -> None:
        """
        重写添加路由方法，在路由中加入url_prefix
        """
        self.url_map[(self.url_prefix or '') + url] = func



# -*- coding: utf_8 -*-
# @Create   : 2021/8/19 14:56
# @Author   : yh
# @Remark   : mx框架db层
from .mqDB import MQ
from .TableDB import TableDB
from .TreeDB import TreeDB


class DB:

    def __init__(self, *args, **kwargs):
        self.tree = TreeDB()
        self.table = TableDB()
        self.mq = MQ()



# -*- coding: utf_8 -*-
# @Create   : 2021/5/26 9:47
# @Author   : yh
# @Remark   : http会话相关的公共方法
from .def_http_code import HttpCode, WeMr


def redirect(session, url: str, params: dict):
    """
    重定向
    :param session: http会话
    :param url: 要重定向的url
    :param params: 参数字典
    :return:
    """
    query = list()

    for k, v in params.items():
        query.append(k + "=" + v)
    if len(query) > 0:
        url = url + "?" + "&".join(query)

    headers = session.GetHttpResponseHead()
    headers.SetStatus(HttpCode.st_302_found)
    headers.SetStatusDescription("OK")
    headers.SetContentType("text/html")
    headers.SetLocation(url)
    headers.SetContentLength(0)
    session.SendHeader()
    session.SendData("")


def send_response(session, data):
    """
    返回响应数据
    :param session: http会话
    :param data: 要发送的数据
    """
    if not session:
        return WeMr.WE_MR_BADREQUEST
    data_len = len(data)
    data_bytes = bytes(data, encoding='utf-8')
    if data_len > 1000:
        session.GetHttpResponseHead().SetContentLength(len(data_bytes))
        session.SendHeader()
        session.SendData(data_bytes)
    elif data_len > 0:
        session.GetHttpResponseHead().SetContentLength(len(data_bytes))
        session.SendHeader()
        session.SendData(data)
    return WeMr.WE_MR_OK



# -*- coding: utf_8 -*-
# @Create   : 2021/6/29 14:06
# @Author   : yh
# @Remark   : mx框架Model层
from pydantic import BaseModel


class Model(BaseModel):
    """
    暂时没有要添加或重写的方法，但是先预留出来
    """

    @classmethod
    def get_fields(cls):
        pass



# -*- coding: utf_8 -*-
# @Create   : 2021/5/11 11:14 上午
# @Author   : yh
# @Remark   : 在此定义所有的table数据库操作方式

TBDB_TABLE_MAXFIELDCOUNT = 4096

BS_TD_MAXUSERNAME = 20
BS_TD_MAXPWD = 30
BS_TD_DBNAME = 30

BS_TD_MAXTALBENAME = 100
# 用于创建表名称索引时节省空间(取中位数的尺寸)
BS_TD_MEDIANTABLENAMELEN = 40

##############################Open DB flag##################################

TBDB_OPDBF_CREATENEW = 0x00000001
TBDB_OPDBF_OPENEXIST = 0x00000002
TBDB_OPDBF_DELETEDB = 0x00000004

###################################################

#################################Create table flag###############################

# 有此标志将增加一个自动增长的ID的列
BS_TDTF_FLAG_AUTOID = 0x00000001
# 固定长度记录模式：在此模式下非固定尺寸的列在创建时必须指定最大尺寸如字符串和二进制数据等
BS_TDTF_FLAG_FIXEDRECORD = 0x00000002
# 当有自动增长iD时如果设置此标志则对自动增长ID建索引并设为主键
BS_TDTF_FLAG_INDEXID = 0x00000004
# 设置此标志位则表示除了数据库自己增加自增加ID外，最少有一个列需要自动增长(在此标记是为了增加插入速度)
BS_TDTF_FLAG_AUTOINCREASE = 0x00000008
# 设置此标志位则统计表占用的页面数
BS_TDTF_FLAG_COUNTPAGE = 0x00000020
# 设置此标志位表示在这个表中除了主键外最少有一个列是有索引的(在此标记是为了增加速度)
BS_TDTF_FLAG_INDEXFIELD = 0x00000040
# 设置此标志位表示这是个主键类型的表，此时将会采取主键加索引去确定行的位置而不用链表形式
BS_TDTF_FLAG_PRIMARYTABLE = 0x00000080
# 设置此标志位表示表中最少有一个字段为VDT_STR或者VDT_BIN类型数据(为了加快插入速度)
BS_TDTF_FLAG_VARFIELD = 0x00000100
# 指明此主键索引为降序模式否则为升序
BS_TDTF_FLAG_PMKI_DESCORDER = 0x00000200
# 设置此标志位表明这是个DYN表
BS_TDTF_FLAG_DYNTABLE = 0x00040000

######condition##############
TBDB_FMC_GREATERTHAN = 0x01
TBDB_FMC_LESSTHAN = 0x02
TBDB_FMC_EQUAL = 0x03
TBDB_FMC_EQUALORGREATERTHAN = 0x04
TBDB_FMC_EQUALORLESSTHAN = 0x05
TBDB_FMC_UNEQUAL = 0x06
#########################

##########condition flag##############

BS_FIELD_COND_USE_END = 0x01
# 此标记为真时表明按创建时间过滤
BS_FIELD_COND_USE_CTIME = 0x02
BS_FIELD_COND_USE_DELETEALL = 0x04

##########################

#######delete table type#########
# 快速删除一个表，这种情况不会释放空间
TBDB_DT_QUICK = 0x01
# 完整删除一个表，会释放表所占的空间
TBDB_DT_RS = 0x02
########################

##########内存中记录集的前缀##########

BS_TDRDSETPREXLEN = 14

######/内存记录集中每条记录的前缀#######

BS_TDRDPREX_LEN = 18

#########/内存中表字段结构##########/
BS_TD_MAXFIELDNAME = 32
BS_TDFF_CREATEINDEX = 0x0001
# 设置此标志则表示此列值不能有重复的
BS_TDFF_UNIQUE = 0x0002
# 设置此标志表示此字段为主键
BS_TDFF_PRIMARYKEY = 0x0004
# 指未此字段为自增id自段,此时数据类型必须为整数型
BS_TDFF_AUTOINCREASE = 0x0008
# 设置此标志指示此列所建的索引为降序，否则为升序
BS_TDFF_INDEX_DESC = 0x0020

BS_TDFIELD_LEN = (7 + BS_TD_MAXFIELDNAME)

##############DYN########################

MSTATUS_OK = 1
MSTATUS_WARNING = 2
MSTATUS_ERROR = 3
MSTATUS_BAD = 4
MSTATUS_DISABLE = 5
MSTATUS_NULL = 0

BS_TBDB_DYNLEN = 28



"""
c++ dyn
struct td_dyn
{
	ui16 uCBLen;
	ui64 tLastChangeTime;
	ui8	uState;
	ui8 uPreState;
	ui64 tLastStateChangeTime;
	ui64 uLastStateKeepTimes;
};

python dyn
dyn = {
    "uCBLen":0,
    "tLastChangeTime":0,
    "uState": 0,
    "uPreState": 0,
    "tLastStateChangeTime": 0,
    "uLastStateKeepTimes": 0
}
"""


"""
c++ bs_tdfield
struct bs_tdfield
{
#define BS_TD_MAXFIELDNAME		32
	i8 name[BS_TD_MAXFIELDNAME];
	i8 uType;
	//设置此标志则对列创建索引
#define BS_TDFF_CREATEINDEX			0x0001
	//设置此标志则表示此列值不能有重复的
#define BS_TDFF_UNIQUE				0x0002
	//设置此标志表示此字段为主键
#define BS_TDFF_PRIMARYKEY			0x0004
	//指未此字段为自增id自段,此时数据类型必须为整数型
#define BS_TDFF_AUTOINCREASE		0x0008
	//设置此标志指示此列所建的索引为降序，否则为升序
#define BS_TDFF_INDEX_DESC			0x0020
	ui16 uFlag;
	//如果是固定长度记录模式，则此值代表非固定长度的数据类型的限定长度，如字符串和二进制数据等
	ui32 uDataLen;

};

python bs_tdfield

bs_tdfield = {
    "name":"name",
    "uType":VDT_STR,
    "uFlag": BS_TDFF_CREATEINDEX,
    "uDataLen": 0
}
"""
# endif



# -*- coding: utf_8 -*-
# @Create   : 2021/5/17 9:47
# @Author   : yh
# @Remark   : 在此编写所有自定义异常类
import json

from .db_def.db_error import error_dict
from .def_http_code import HttpCode


class MxBaseException(Exception):
    """
    异常基类
    """
    state_code = HttpCode.st_200_ok  # 默认响应码

    def __init__(self, msg: str = None) -> None:
        """
        :param msg: 错误提示信息
        """
        super().__init__()
        self.msg = msg


class CError(MxBaseException):
    """
    c++方法调用异常
    """

    def __str__(self):
        if not self.msg:
            msg = 'c++方法调用异常'
        else:
            msg = str(self.msg) + ': c++方法调用异常'

        return json.dumps({'status': 'failed', 'errmsg': msg}, ensure_ascii=False)


class ModelCheckError(MxBaseException):
    """
    model层字段验证异常
    """

    def __str__(self):
        if not self.msg:
            msg = '模型字段验证错误'
        else:
            msg = str(self.msg) + ': 模型字段验证错误'

        return json.dumps({'status': 'failed', 'errmsg': msg}, ensure_ascii=False)


class ParamsError(MxBaseException):
    """
    参数异常
    """

    def __str__(self):
        if not self.msg:
            msg = '参数错误'
        else:
            msg = str(self.msg) + ': 参数错误'

        return json.dumps({'status': 'failed', 'errmsg': msg}, ensure_ascii=False)


class DBError(MxBaseException):
    """
    数据库调用异常
    """

    def __init__(self, err_code: int, msg: str = None):
        """
        :param err_code: 数据库错误码
        :param msg: 错误提示信息
        """
        super().__init__()
        self.err_code = err_code  # 数据库错误码
        self.msg = msg

    def __str__(self):
        if not self.msg:
            msg = '数据库错误，错误码: %s, 错误原因: %s' % (self.err_code, error_dict.get(self.err_code))
        else:
            msg = '%s: 数据库错误, 错误码: %s, 错误原因: %s' % (self.msg, self.err_code, error_dict.get(self.err_code))

        return json.dumps({'status': 'failed', 'errmsg': msg}, ensure_ascii=False)


class DataError(MxBaseException):
    """
    数据异常：通常为数据不符合要求
    """

    def __str__(self):
        if not self.msg:
            msg = '数据错误'
        else:
            msg = '%s: 数据错误' % self.msg

        return json.dumps({'status': 'failed', 'errmsg': msg}, ensure_ascii=False)


class NotFoundError(MxBaseException):
    """
    url路径未找到
    """
    state_code = HttpCode.st_404_not_found

    def __str__(self):
        if not self.msg:
            msg = '未知的url请求'
        else:
            msg = str(self.msg) + ': 未知的url请求'

        return json.dumps({'status': 'failed', 'errmsg': msg}, ensure_ascii=False)


class HTTPMethodError(MxBaseException):
    """
    请求方式异常
    """

    def __str__(self):
        if not self.msg:
            msg = '错误的请求方法'
        else:
            msg = '%s: 错误的请求方法' % self.msg

        return json.dumps({'status': 'failed', 'errmsg': msg}, ensure_ascii=False)


class FileError(MxBaseException):
    """
    文件异常
    """

    def __str__(self):
        if not self.msg:
            msg = '文件错误'
        else:
            msg = '%s: 文件读取错误' % self.msg

        return json.dumps({'status': 'failed', 'errmsg': msg}, ensure_ascii=False)


class AuthError(MxBaseException):
    """
    权限异常
    """

    def __str__(self):
        if not self.msg:
            msg = '权限错误'
        else:
            msg = '%s: 权限错误' % self.msg

        return json.dumps({'status': 'failed', 'errmsg': msg}, ensure_ascii=False)



# -*- coding: utf_8 -*-
# @Create   : 2021/9/13 9:24
# @Author   : yh
# @Remark   : 数据库操作方法基类，用于执行数据库方法及返回值校验
from typing import Union, Any, List

from .db_def.def_type import type_map
from .exception import DBError, DataError
from superbsapi import *

from .db_def.db_error import BS_NOERROR


class BaseDB:

    def __init__(self, host=None, port=None) -> None:
        """
        初始化属性
        :param host: 主机
        :param port: 端口
        """
        self.__chl = CBSHandleLoc()
        self.__handle = None
        try:
            from utils.conf.mxconfig import MxConfig
            self.host = host or MxConfig.HOST
            self.port = port or MxConfig.PORT
        except (ModuleNotFoundError, ImportError, AttributeError):
            self.host = host or '127.0.0.1'
            self.port = port or 8123
        self.__cls_value(self.host, self.port)

    @staticmethod
    def return_value(res: tuple) -> Union[int, str, tuple, None]:
        """
        用于直接返回结果集的函数，处理其返回值

        :param res: 错误信息及函数返回的数据
        :return: 返回信息
        """
        msg = res[0] if isinstance(res, tuple) else res
        if msg != BS_NOERROR:
            raise DBError(msg)
        if res:
            return res[1] if len(res) == 2 else res[1:]
        return None

    def exec_tree(self, operate: str, *args: Any, **kwargs: Any) -> Union[int, str, tuple, None]:
        """
        用于执行Tree开头的函数

        :param operate: 使用的函数
        :param args kwargs: 函数所使用的变量
        :return: 执行结果
        """
        func = getattr(self.__chl, operate)
        res = func(*args, **kwargs)
        return self.return_value(res)

    def exec_bs(self, operate: str, *args: Any, **kwargs: Any) -> Union[int, str, tuple, None]:
        """
        用于执行bs开头的函数

        :param operate: 使用的函数
        :param args kwargs: 函数所使用的变量
        :return: 执行结果
        """
        res = eval(operate)(self.__chl.GetConfHandle(), *args, **kwargs)
        return self.return_value(res)

    @classmethod
    def exec_class(cls, operate: str, *args, **kwargs):
        """
        用于执行类方法

        :param operate: 使用的函数
        :param args kwargs: 函数所使用的变量
        :return: 执行结果
        """
        func = getattr(CBSHandleLoc, operate)
        res = func(*args, **kwargs)
        return cls.return_value(res)

    def exec(self, operate: str, *args, **kwargs):
        """
        用于执行直接连接的函数（不在CBSHandleLoc类中的连接函数）

        :param operate: 使用的函数
        :param args kwargs: 函数所使用的变量
        :return: 执行结果
        """
        res, self.__handle = eval(operate)(*args, **kwargs)
        return self.return_value(res)

    def exec_handle(self, operate: str, *args, **kwargs):
        """
        用于使用上面连接所产生handle的函数

        :param operate: 使用的函数
        :param args kwargs: 函数所使用的变量
        :return: 执行结果
        """
        if self.__handle:
            res = eval(operate)(self.__handle, *args, **kwargs)
        else:
            raise DataError('找不到句柄，请先连接到数据库')
        return self.return_value(res)

    @staticmethod
    def _check_item(item: tuple) -> int:
        """
        校验item是否符合数据要求并返回value type
        """
        if not isinstance(item, tuple) or len(item) not in [2, 3]:
            raise DataError('错误的数据类型，items列表中的数据应为元组且长度为2或3')
        value_type = type_map.get(item[2]) if len(item) == 3 else type_map.get(
            type(item[1]).__name__)
        if not value_type:
            raise DataError('无法获取到value的类型！如果您未手动传入类型或确信传入类型正确，请于type_map中添加此类型')
        value_type = type_map[item[2]] if len(item) == 3 else type_map[type(item[1]).__name__]
        return value_type

    def _generation_items(self, items: Union[List[tuple], dict]) -> list:
        """
        生成符合数据库插入格式的数据
        """
        data_list = list()
        if isinstance(items, list):
            for item in items:
                data_list.append({'name': item[0], 'value': item[1], 'valuetype': self._check_item(item)})
        elif isinstance(items, dict):
            for k, v in items.items():
                if type_map.get(type(v).__name__):
                    data_list.append({'name': k, 'value': v, 'valuetype': type_map[type(v).__name__]})
                else:
                    raise DataError('未知的value_type, 类型：%s ,值：%s' % (str(type(v)), v))
        else:
            raise DataError('错误的数据类型，items应为列表或字典')

        return data_list

    @classmethod
    def __cls_value(cls, host=None, port=None) -> None:
        """
        将实例属性添加到类属性
        :param host: 主机
        :param port: 端口
        """
        cls.host = host
        cls.port = port

    @staticmethod
    def _check_conn_params(cls, host, port) -> tuple:
        """
        获取并检查连接参数并返回
        :param cls: 类
        :param host: 主机ip
        :param port: 端口
        """
        host = host or getattr(cls, 'host', None)
        port = port or getattr(cls, 'port', None)

        if not host or not port:
            raise DataError('无法获取主机或端口，请实例化类或直接传递参数')

        return host, port



# -*- coding: utf_8 -*-
# @Author : yh
# @Time : 2020/11/5 16:18
# @Remark: 存放Tree db数据库的操作方法，详情见TreeModel使用手册
import logging
from typing import List, Any, Union

from superbsapi import CBSHandleLoc

from .BaseDB import BaseDB
from .exception import DBError, DataError
from .db_def.def_type import *
from .db_def.def_tree import *

symbol_map = {
    'e': TRDB_FMC_EQUAL,  # 等于
    'ne': TRDB_FMC_UNEQUAL,  # 不等于
    'gt': TRDB_FMC_GREATERTHAN,  # 大于
    'lt': TRDB_FMC_LESSTHAN,  # 小于
    'gte': TRDB_FMC_EQUALORGREATERTHAN,  # 大于等于
    'lte': TRDB_FMC_EQUALORLESSTHAN,  # 小于等于

    'like': TRDB_FMC_LIKE,  # 模糊匹配，使用 * 做为通配符
    'nclike': TRDB_FMC_NOCASE_LIKE,  # 模糊匹配，不区分大小写
    'in': TRDB_FMC_RANGE,  # 范围查找（枚举）
    'between': TRDB_FMC_RANGE  # 范围查找（范围）
    # 其他的类型，以后用到了再添加
}


class TreeDB(BaseDB):

    def begin(self) -> int:
        """
        开启事务
        """
        return self.exec_tree('Treedb_BeginTransaction')

    def commit(self) -> int:
        """
        提交事务
        :return:
        """
        return self.exec_tree('Treedb_CommitTransaction')

    def rollback(self) -> int:
        """
        回滚事务
        """
        return self.exec_tree('Treedb_RollbackTransaction')

    def create_index(self, name, name_type, flag=0):
        """
        创建索引
        :param name: 要建立索引的字段名称
        :param name_type: 字段类型
        :param flag: 默认升序，设置BS_RITF_INDEX_DESC则为降序
        :return:
        """
        return self.exec_bs('bs_treedb_create_index', [{'name': name, 'uFlag': flag,
                                                        'uType': type_map.get(name_type) or name_type}])

    def delete_index(self):
        """
        删除索引
        :return:
        """

        # TODO 用到了再写吧
        # return self.exec_bs('bs_treedb_delete_index')

    def open(self, main_key: str, sub_key: str = None, host: str = None, file: str = 'base',
             main_key_pwd: str = '', flag: int = None, port: int = None, path_flag: bool = False) -> "TreeDB":
        """
        打开数据库键 [只打开主键或子键 / 同时打开主键和子键（子键路径）]
        eg: db = TreeModel()
            db.open('MXSE', '1.SD', file='ccubase', host='127.0.0.1')
            
        :param main_key: 主键名
        :param sub_key: 子键名
        :param host: 主机名
        :param file: 数据库名
        :param main_key_pwd: 主键密码
        :param flag: 打开风格
        :param port: 端口号
        :param path_flag: 如果路径上有子键不存在是否自动创建
        :return: 类对象
        """
        host = host or self.host
        port = port or self.port

        if not flag:
            if sub_key:
                flag = TRDB_OPKF_OPENEXIST if '\\' in sub_key else TRDB_OPKF_DOTPATH
            else:
                flag = TRDB_OPKF_OPENEXIST if '\\' in main_key else TRDB_OPKF_DOTPATH

        if sub_key:
            try:
                self.exec_tree('Treedb_ReopenMainKey', sub_key, flag, path_flag, main_key, main_key_pwd, file)
            except DBError:
                try:
                    self.__chl = CBSHandleLoc()
                    self.exec_tree('Treedb_Alloc', host, file, main_key, main_key_pwd, TRDB_OPKF_OPENEXIST, port)
                    logging.warning('正在重新连接数据库并打开主键[%s]-子键[%s]' % (main_key, sub_key))
                    self.exec_tree('Treedb_ReopenSubKey', sub_key, flag)
                except DBError as e:
                    raise DBError(e.err_code, '打开主键[%s]-子键[%s]时' % (main_key, sub_key))
        else:
            try:
                self.exec_tree('Treedb_ReopenSubKey', main_key, flag)
            except DBError:
                try:
                    self.exec_tree('Treedb_ReopenMainKey', '', flag, path_flag, main_key, main_key_pwd, file)
                except DBError:
                    try:
                        self.__chl = CBSHandleLoc()
                        self.exec_tree('Treedb_Alloc', host, file, main_key, main_key_pwd, TRDB_OPKF_OPENEXIST, port)
                        logging.warning('正在重新连接tree数据库并打开主键[%s]' % main_key)
                    except DBError as e:
                        raise DBError(e.err_code, '打开主键[%s]时' % main_key)
        return self

    @classmethod
    def main_keys(cls, file: str, host: str = None, port: int = None) -> list:
        """
        获取数据库下所有的主键
        eg: TreeDB.main_keys(file='master', host='127.0.0.1', port=8123)

        :param file: 数据库名
        :param host: 主机
        :param port: 端口
        """

        host, port = cls._check_conn_params(cls, host, port)

        res_list = list()
        cls.exec_class('Treedb_GetAllMainKeyNames', res_list, host, file, port)
        return res_list

    def sub_keys(self) -> list:
        """
        获取当前键下所有的子键名
        eg: sub_keys = db.open('MXSE', '1.SD', file='ccubase').sub_keys()
        
        :return: 子键列表
        """
        res_list = list()
        self.exec_tree('Treedb_GetAllSubKeys', res_list)
        return res_list

    def sub_items(self, key_list=None, prop_list=None) -> dict:
        """
        获取当前键下，指定子键，指定属性的 key-value键值对
        eg: sub_items = db.open('MXSE', '1.SD', file='ccubase').sub_items(
                                key_list=['1.SD.1', '1.SD.2'], prop_list='_target_ip')
        <<<!!! 当key_list为空的时候会获取全部，需要自行注意 !!!>>>
            
        :param key_list: 要获取的子键列表
        :param prop_list: 要获取的属性列表
        :return: 子键及其属性字典
        """
        res_dict = dict()
        if isinstance(key_list, str):
            key_list = [key_list]
        if isinstance(prop_list, str):
            prop_list = [prop_list]

        self.exec_bs('bs_treedb_get_subkey_properties', key_list or [], prop_list or [], res_dict)
        return res_dict

    def items(self, prop_list=None) -> dict:
        """
        获取当前键下、指定属性的 key-value键值对, prop_list为空会获取所有属性
        eg: items = db.open('MXSE', '1.SD', file='ccubase').items(['max', 'mx_updatedatatime'])
        
        :param prop_list: 要获取属性
        """
        prop_list = [prop_list] if isinstance(prop_list, str) else prop_list

        res_dict = dict()
        self.exec_bs('bs_treedb_get_properties', prop_list if prop_list else [], res_dict)
        return res_dict

    def props(self) -> list:
        """
        获取所有属性名
        eg: props = db.open('MXSE', '1.SD', file='ccubase').props()

        :return: 属性名列表
        """
        res_list = list()
        self.exec_tree('Treedb_GetAllPropertyNames', res_list)
        return res_list

    def fetchone_value(self, prop_name: str) -> Any:
        """
        获取一个属性的值
        eg: max = db.open('MXSE', '1.SD', file='ccubase').fetchone_value('max')

        :param prop_name: 要获取的属性名称
        :return: 属性值
        """
        return self.exec_tree('Treedb_GetProperty', prop_name)

    @classmethod
    def insert_file(cls, file_name: str, host: str = None, port: int = None) -> int:
        """
        插入数据库

        :param file_name: 数据库名称
        :param host: 主机ip
        :param port: 端口
        """

        host, port = cls._check_conn_params(cls, host, port)
        return cls.exec_class('Treedb_CreateFile', host, file_name, port)

    @classmethod
    def insert_main_key(cls, main_key: str, file: str, main_pwd: str = '', host: str = None, port: int = None) -> int:
        """
        插入主键
        eg: db.insert_main_key('test', host='127.0.0.1', file='IOT')

        :param main_key: 主键名
        :param file: 文件名
        :param main_pwd: 主键密码
        :param host: 主机ip
        :param port: 端口
        """

        host, port = cls._check_conn_params(cls, host, port)
        return cls.exec_class('Treedb_CreateMainKey', host, file, main_key, main_pwd, port)

    def insert_sub_keys(self, sub_keys: Union[str, list], flag: int = TDDB_OPKF_CREATEDYNKEY) -> Union[str, list]:
        """
        批量插入子键
        eg: db.open('MXSE', '1.SD', file='ccubase').insert_sub_keys(['test1', 'test2'])

        :param sub_keys: 子键列表
        :param flag: 打开风格
        """
        if isinstance(sub_keys, str):
            return self.exec_tree('Treedb_InsertSubKey', sub_keys, flag)
        else:
            res = []
            for sub_key in sub_keys:
                res.append(self.exec_tree('Treedb_InsertSubKey', sub_key, flag))
            return res

    def insert_item(self, prop: str, value: Any, value_type: str = None, overwrite: bool = True) -> int:
        """
        插入一条数据
        eg: db.open('MXSE', '1.SD', file='ccubase').insert_item('mxlabel', 'test')

        :param prop: 键
        :param value: 值
        :param value_type: 指定的数据类型，不传自动获取
        :param overwrite: 是否覆盖原值
        """
        value_type = type_map.get(value_type) or type_map.get(type(value).__name__)
        if not value_type:
            raise DataError('未知的value_type, 类型：%s ,值：%s' % (str(type(value)), value))
        return self.exec_bs('bs_treedb_insert_property', prop, value, value_type, overwrite)

    def insert_items(self, items: Union[List[tuple], dict], overwrite=True) -> int:
        """
        批量插入数据
        eg: db.open('MXSE', '1.SD', file='ccubase').insert_items([('mxlabel', 'test'), ('mxid', 123)])
            db.open('MXSE', '1.SD', file='ccubase').insert_items({'mxlabel': 'test', 'mxid': 123})

        :param items: 插入的数据
        :param overwrite: 是否覆盖原值
        """

        return self.exec_bs('bs_treedb_insert_propertys', self._generation_items(items), overwrite) if items else 0

    def insert_key_items(self, items: Union[List[tuple], dict], key: str = None) -> str:
        """
        批量插入子键及k,v
        eg: db.open('MXSE', '1.SD', file='ccubase').insert_key_items(
                                                    items=[('mxlabel', 'test'), ('mxid', 123)], key='key1')
            db.open('MXSE', '1.SD', file='ccubase').insert_key_items(
                                                    items={'mxlabel': 'test', 'mxid': 123}, key='key1')

        :param items: 插入的数据, 如果元组传入三个元素，将第三个元素当作数据类型（见文首type_map），如果只传入两个，则自动获取数据类型
        :param key: 子键名称,不传自动生成
        :return: 生成的子键
        """

        return self.exec_bs('bs_treedb_insert_key_and_properties', key or '', TRDB_OPKF_OPENEXIST,
                            self._generation_items(items), True) if items else 0

    def delete(self, keys: Union[list, str, None] = None) -> None:
        """
        删除 单个子键/子键列表/自身
        eg: db.open('MXSE', '1.SD', file='ccubase').delete()

        :param keys: 要删除的子键。
        <<<!!! 值为空时会删除自身 !!!>>>
        <<<!!! 值为空时会删除自身 !!!>>>
        <<<!!! 值为空时会删除自身 !!!>>>
        """
        if isinstance(keys, list):
            for key in keys:
                self.exec_tree('Treedb_DeleteSubKey', key)
        elif isinstance(keys, str):
            self.exec_tree('Treedb_DeleteSubKey', keys)

        if keys is None:
            self.exec_bs('bs_treedb_delete_key')

    def delete_prop(self, prop: Union[str, None] = None) -> None:
        """
        删除一个属性、所有属性
        eg: db.open('MXSE', '1.SD', file='ccubase').delete_prop()

        :param prop: 要删除的属性名
        """
        if prop:
            self.exec_tree('Treedb_DeleteProperty', prop)
        else:
            self.exec_tree('Treedb_DeleteAllProperty')

    def rename(self, old_key: str, new_key: str) -> int:
        """
        重命名一个子键、自身
        eg: db.open('MXSE', '1.SD', file='ccubase').rename('1.SD.1', '1.SD.111')

        :param old_key: 要重命名的子键
        :param new_key: 新的名字
        （重命名自身的时候只需要传递新的名字）
        """
        if new_key:
            return self.exec_tree('Treedb_RenameSubKey', old_key, new_key)
        else:
            return self.exec_tree('Treedb_RenameThisKey', old_key)

    def rename_prop(self, old_prop: str, new_prop: str) -> int:
        """
        重命名一个属性
        eg: db.open('MXSE', '1.SD', file='ccubase').rename_prop('mxlabel', 'malabel-rename')

        :param old_prop: 要重命名的属性
        :param new_prop: 新的属性名
        """
        return self.exec_tree('Treedb_RenameProperty', old_prop, new_prop)

    def filter(self, **kwargs) -> tuple:
        """
        根据属性筛选符合条件的子键（可分页显示）

        :param kwargs: 传入的查询条件
                        page_size：每页条数
                        page_index：第几页
                        order_by：排序字段
                        is_desc：是否反序
                        default_expression：手动传入条件关系，默认关系为 and
        :return: 总条数，查询结果
        """

        try:
            page_size = kwargs.pop('page_size')
            assert page_size, '未获取到数据！'
        except (KeyError, AssertionError):
            page_size = 0
        try:
            page_index = kwargs.pop('page_index')
            assert page_index, '未获取到数据！'
        except (KeyError, AssertionError):
            page_index = 1
        try:
            order_by = kwargs.pop('order_by')
            assert order_by, '未获取到数据！'
        except (KeyError, AssertionError):
            order_by = ''
        try:
            is_desc = kwargs.pop('is_desc')
        except KeyError:
            is_desc = False
        try:
            default_expression = kwargs.pop('default_expression')
            assert default_expression, '未获取到数据！'
        except (KeyError, AssertionError):
            default_expression = None
        args, range_conditions = list(), dict()

        i = 0
        expression_list = list()

        for key, value in kwargs.items():
            try:
                key, symbol = key.rsplit('__', 1)
                if symbol not in symbol_map:
                    raise DataError('查询操作错误！正确操作包含：gt、lt等')
            except ValueError:
                symbol = 'e'

            temp = {'key': key, 'value_type': type(value).__name__, 'symbol': symbol, 'value': value}

            if symbol == 'in':
                j = 0
                if not isinstance(value, list):
                    raise DataError('查询格式错误！正确示例：a__in=[1, 3, 4, 5]')
                if len(value) == 0:
                    raise DataError('使用in时列表不能为空')
                for v in value:
                    temp = {'key': key, 'value_type': type(v).__name__, 'symbol': 'e', 'value': v}
                    args.append(temp)
                    if j == 0:
                        expression_temp = '(%s' % i
                    else:
                        expression_temp += ' or ' + str(i)
                    i += 1
                    j += 1
                expression_temp += ')'
                expression_list.append(expression_temp)
            elif symbol == 'between':
                if not isinstance(value, list) or len(value) not in [2, 3]:
                    raise DataError('查询格式错误！正确示例：a__between=[1, 3]，详情见TreeModel使用手册')
                temp['range_conditions'] = {'vLiData': value[0], 'vEnd': value[1]}
                temp['value_type'] = value[2] if len(value) == 3 else type(value[0]).__name__
                expression_list.append(' %s ' % i)
                args.append(temp)
                i += 1
            else:
                expression_list.append(' %s ' % i)
                args.append(temp)
                i += 1

        default_query_conditions = list()
        for arg in args:
            if arg['symbol'] not in symbol_map:
                raise DataError('查询操作错误！正确操作包含：gt、lt等，详情见TreeModel使用手册')
            if arg['value_type'] not in type_map:
                raise DataError('查询数据类型！正确操作包含：str、int等，详情见TreeModel使用手册')
            data = {'name': arg['key'], 'nCondition': symbol_map[arg['symbol']], 'vLiData': arg['value'],
                    'vLiDataType': type_map[arg['value_type']]}
            if arg.get('range_conditions'):
                data.update(arg['range_conditions'])
            default_query_conditions.append(data)

        res_list = list()
        res = self.exec_bs('bs_treedb_query_subkey_by_condition_ex', default_query_conditions,
                           default_expression if default_expression else ' and '.join(expression_list),
                           res_list, page_size, (page_index - 1) * page_size, order_by, is_desc)

        return res, res_list



# -*- coding: utf_8 -*-
# @Create   : 2021/5/11 11:14 上午
# @Author   : yh
# @Remark   : 在此定义所有的消息队列操作方式

# 定义所有打开方式
BSMQ_OF_OPENEXIST = 0x00000001  # 打开已存在的mq
BSMQ_OF_CREATENEW = 0x00000002  # 创建新的mq
BSMQ_OF_OPENMULTI = 0x00000004

# 定义创建mq的风格
BSMQ_OT_COMMONMQ = 0x01  # 普通队列
BSMQ_OT_PRIORITYMQ = 0x02  # 优先队列
BSMQ_OT_TEMPMQ = 0x04
BSMQ_OT_ENCRYPT = 0x08
BSMQ_OT_INDEXRECORD = 0x10

BS_MQ_MAXNAME = 100
BS_MQ_MAXPWD = 20

# 表示最多支持多少个优先级
BS_MQ_MAXPRIORITY = 9
# 普通优先级
BS_MQ_COMMONPRIORITY = (BS_MQ_MAXPRIORITY / 2 + 1)

BS_MQ_MAXRECORDID = 30

# 从mq取数的等待时间设置为无限等待
BS_TIMER_INFINITE = 0xFFFFFFFF



# -*- coding: utf_8 -*-
# @Create   : 2021/9/13 9:12
# @Author   : yh
# @Remark   : 存放Table db数据库的操作方法
import datetime
import logging
from typing import Type, List, Any

from superbsapi import CBSHandleLoc

from .BaseDB import BaseDB
from .db_def.def_table import *
from .db_def.def_type import type_map, type_c_python, type_c_str
from .exception import DBError, DataError
from . import Model

symbol_map = {
    'e': TBDB_FMC_EQUAL,  # 等于
    'ne': TBDB_FMC_UNEQUAL,  # 不等于
    'gt': TBDB_FMC_GREATERTHAN,  # 大于
    'lt': TBDB_FMC_LESSTHAN,  # 小于
    'gte': TBDB_FMC_EQUALORGREATERTHAN,  # 大于等于
    'lte': TBDB_FMC_EQUALORLESSTHAN,  # 小于等于
    'between': TBDB_FMC_GREATERTHAN
    # 其他的类型，以后用到了再添加
}


class TableDB(BaseDB):

    def __init__(self, host=None, port=None):
        super().__init__(host, port)
        self.__table = None

    def open(self, file: str, table: str = None, host: str = None, port: str = None):
        """
        打开table数据库
        :param file: 数据库名称
        :param table: 数据库表名
        :param host: 主机ip
        :param port: 端口
        """
        host = host or self.host
        port = port or self.port

        try:
            self.exec_tree('Tabledb_ReopenDb', file)
        except DBError:
            try:
                self.__chl = CBSHandleLoc()
                self.exec_tree('Tabledb_Alloc', host, file, port)
                logging.warning('正在重新连接数据库并打开table[%s]' % file)
            except DBError as e:
                raise DBError(e.err_code, '打开数据库[%s]失败' % file)

        self.__table = table

        return self

    def table_list(self) -> list:
        """
        获取所有数据库表名
        """
        res = list()
        self.exec_bs('bs_tabledb_get_all_tablenames', res)

        return res

    def fields(self, table_name: str = None) -> dict:
        """
        获取表的所有字段及其属性
        :param table_name: 要获取字段类型的表名
        :return:
        """
        res = list()
        table = table_name or self.__table
        if not table:
            raise DataError('找不到table，请打开或传入表')

        self.exec_bs('bs_tabledb_get_fields', table, res)
        return {i['name']: i for i in res}

    def create_table(self, table: str, model: Type[Model], flag=None, **kwargs) -> None:
        """
        新建表
        :param table: 要创建的数据库名称
        :param model: 要创建的表，如果您不显式的声明字段长度，则默认长度为200
        :param flag: 创建风格，不传会自动判断
        :param kwargs: 附加参数
        """
        flag = 0x00000000
        if kwargs.get('auto_id'):
            flag += BS_TDTF_FLAG_AUTOID
        if kwargs.get('auto_id_index'):
            flag += BS_TDTF_FLAG_INDEXID
        if kwargs.get('primary_key'):
            flag += BS_TDTF_FLAG_PRIMARYTABLE
        if kwargs.get('desc'):
            flag += BS_TDTF_FLAG_PMKI_DESCORDER
        if kwargs.get('dyn'):
            flag += BS_TDTF_FLAG_DYNTABLE
        if not flag:
            flag = BS_TDTF_FLAG_AUTOID

        field_list = list()
        for field in model.__fields__.values():
            if not type_map.get(field.type_.__name__):
                raise DataError('不支持此解析字段的类型，字段名: %s, 解析到的类型: %s' % (field.name, field.type_.__name__))

            field_list.append({"name": field.name, "uType": type_map[field.type_.__name__],
                               "uDataLen": field.type_.max_length if hasattr(field.type_, 'max_length') else 200})

        self.exec_bs('bs_tabledb_create_table', table, field_list, flag)

    def insert(self, table: str, data: List[tuple]) -> None:
        """
        向表中插入数据
        :param table: 表名
        :param data: 要insert的数据
        """
        if not isinstance(data, list):
            raise DataError('错误的数据类型，data应为列表，其中的元素应为元组')

        insert_list = list()

        for item in data:
            insert_list.append({'name': item[0], 'value': item[1], 'valuetype': self._check_item(item)})

        self.exec_bs('bs_tabledb_insert_record', table, insert_list)

    @staticmethod
    def __conversion_data(value: Any, value_type: int) -> Any:
        """
        校验数据类型是否合法并value到对应转换类型
        """
        if not type_c_str.get(value_type):
            raise DataError('无法理解的C++数据类型：没有定义此类型：%s' % value_type)
        if not type_c_python.get(value_type):
            raise DataError('从C++数据类型转为Python类型时出错，没有定义转换关系：%s' % type_c_str[value_type])

        try:
            value = eval(type_c_python[value_type])(value)
        except ValueError:
            raise DataError('无法转换数据类型：将传入的数据类型%s转为C++类型%s（实际转为Python类型%s）' % (
                type(value).__name__, type_c_str[value_type], type_c_python[value_type]))

        return value

    class __TimeDelta(object):
        def __init__(self, **kwargs):
            self._query = dict()
            for k, v in kwargs.items():
                if k in ['days', 'seconds', 'microseconds', 'milliseconds', 'minutes', 'hours', 'weeks'] and isinstance(
                        v, (float, int)):
                    self._query[k] = v

        def res(self):
            return self._query

    def record_by_time(self, table_name: str = None, start_time: int = None, end_time: int = None, count=0,
                       **kwargs) -> list:
        """
        根据时间获取记录
        :param table_name: 要获取的表名
        :param start_time: 开始时间戳
        :param end_time: 结束时间戳
        :param count: 要获取的条数，为0则获取所有
        :param kwargs: 时间参数 ['days', 'seconds', 'microseconds', 'milliseconds', 'minutes', 'hours', 'weeks']
        :return:
        """
        self.__table = table_name or self.__table
        res = list()
        if not self.__table:
            raise DataError('找不到table，请打开或传入表')
        if start_time is None and end_time is None:
            now = datetime.datetime.now()
            end_time = int(now.timestamp())
            start_time = int((now - datetime.timedelta(**self.__TimeDelta(**kwargs).res())).timestamp())
        elif start_time is not None and end_time is None:
            end_time = int((datetime.datetime.fromtimestamp(start_time) + datetime.timedelta(
                **self.__TimeDelta(**kwargs).res())).timestamp())
        elif start_time is None and end_time is not None:
            start_time = int((datetime.datetime.fromtimestamp(end_time) - datetime.timedelta(
                **self.__TimeDelta(**kwargs).res())).timestamp())

        self.exec_tree('Tabledb_SelectRecordsByCTime', self.__table, count, start_time, end_time, res)
        return res

    def filter(self, table_name: str = None, count: int = 0, **kwargs) -> List[dict]:
        """
        根据条件查询符合的记录
        :param table_name: 要查询的表名（已经在open方法中打开表以后就不用再传入）
        :param count: 要获取的总条数
        :param kwargs: 查询条件
        """
        self.__table = table_name or self.__table

        if len(kwargs) > 1 or not kwargs:
            raise DataError('每次有且只能查询一个条件')
        if not self.__table:
            raise DataError('没有打开任何表')

        key = next(iter(kwargs))
        value = kwargs[key]
        table_fields = self.fields()

        try:
            key, symbol = key.rsplit('__', 1)
            if symbol not in symbol_map:
                raise DataError('查询操作错误！正确操作包含：gt、lt等')
        except ValueError:
            symbol = 'e'
        if not table_fields.get(key):
            raise DataError('数据库中没有此字段：%s' % key)

        query_conditions = list()
        value_type = table_fields[key]['uType']
        if symbol == 'between':
            if not isinstance(value, list) or len(value) != 2:
                raise DataError('between操作有且仅有两个条件')
            for i in value:
                query_conditions.append({'val': self.__conversion_data(i, value_type), 'type': value_type})
        else:
            query_conditions.append({'val': self.__conversion_data(value, value_type), 'type': value_type})

        res_list = list()
        self.exec_tree('Tabledb_SelectRecordsByField', self.__table, count, key, symbol_map[symbol], query_conditions,
                       res_list)

        return res_list



# -*- coding: utf_8 -*-
# @Create   : 2021/5/27 11:37
# @Author   : yh
# @Remark   : 主入口文件
import json
import sys
import typing as t
from itertools import chain

from pydantic import ValidationError

from .HTTPMethod import send_response
from .base import BaseMx
from .exception import NotFoundError, CError, MxBaseException
from .view import Request, Response

if t.TYPE_CHECKING:
    from .module import Module


class Mx(BaseMx):
    """
    app对象类
    """

    def __init__(self, **options: t.Any):
        super().__init__(**options)
        self.after_request_funcs = dict()
        self.before_request_funcs = dict()

    def before_request(self, f):
        self.before_request_funcs.setdefault(None, []).append(f)
        return f

    def after_request(self, f):
        self.after_request_funcs.setdefault(None, []).append(f)
        return f

    @staticmethod
    def set_response(request: "Request") -> "Request":
        """
        添加响应信息
        """
        try:
            request.response_headers_cls.SetStatus(request.status_code)
        except TypeError:
            raise CError('session.GetHttpResponseHead().SetStatus()')
        try:
            request.response_headers_cls.SetContentType(request.response_content_type)
        except TypeError:
            raise CError('session.GetHttpResponseHead().SetContentType()')
        return request

    @staticmethod
    def package_data(data: t.Any, callback: str) -> json:
        """
        打包响应数据
        :param data: 视图返回的数据
        :param callback: 回调标识
        :return: 打包后的数据
        """
        if isinstance(data, tuple):
            data = json.dumps({'status': 'success', 'errmsg': data[0], 'data': data[1] if len(data) == 2 else data[1:]},
                              ensure_ascii=False)
        else:
            data = json.dumps({'status': 'failed', 'errmsg': data}, ensure_ascii=False)

        if callback:
            data = '%s(%s)' % (callback, data)

        return data

    def response_handle(self, request: "Request", res: t.Any) -> tuple:
        """
        响应处理
        :param res: 视图返回的数据
        :param request: 请求类
        :return:
        """
        request = self.set_response(request)
        if isinstance(res, Response):
            res = str(res)
        else:
            res = self.package_data(res, request.callback)
        return request.session, res

    def register_module(self, module: "Module", **options: t.Any) -> None:
        """
        注册模块
        """
        for k, v in module.url_map.items():
            self.url_map[k] = v

    def get_func(self, session) -> t.Callable:
        """
        从url_map获取对应的函数
        :return:
        """
        self.session_handler = Request(session)
        url = self.session_handler.url
        if url is None:
            url = ''
        url = '/' + url if not url.startswith('/') else url
        return self.url_map.get(url)

    def mx_base_exception_handler(self) -> None:
        """
        对捕获到的MxBaseException进行处理
        """
        error_type, error_value, error_traceback = sys.exc_info()
        self.session_handler = self.set_response(self.session_handler)

        error = error_type(error_value)
        self.session_handler.session.GetHttpResponseHead().SetStatus(error.state_code)

        send_response(self.session_handler.session, '%s(%s)' % (
            self.session_handler.callback, str(error_value)) if self.session_handler.callback else str(
            error_value))

    def validation_error_handler(self):
        """
        对捕获到的ValidationError进行处理
        """
        error_type, error_value, error_traceback = sys.exc_info()
        self.session_handler = self.set_response(self.session_handler)

        error = json.dumps({'status': 'failed',
                            'errmsg': error_value.model.__name__ + ': 模型字段校验失败, ' +
                                      str([{' -> '.join(str(e) for e in error['loc']): error['msg']}
                                           for error in error_value.errors()])},
                           ensure_ascii=False)

        send_response(self.session_handler.session,
                      '%s(%s)' % (self.session_handler.callback, error) if self.session_handler.callback else error)

    def run_func(self, session) -> None:
        """
        运行url对应的函数、异常处理
        """
        func = self.get_func(session)

        # funcs = self.before_request_funcs.get(None, ())
        # funcs = chain(funcs, self.before_request_funcs[])
        #
        # for func in funcs:
        #     func(self.session_handler)

        if func:
            try:
                resp_cls, res = func(self.session_handler)
                send_response(*self.response_handle(resp_cls, res))
            except MxBaseException:
                self.mx_base_exception_handler()
            except ValidationError:
                self.validation_error_handler()

        else:
            raise NotFoundError(self.url_map)

    def __call__(self, session):
        """
        处理请求
        将所有处理放在其它方法中，方便他人进行中间件重写
        """
        self.run_func(session)



# -*- coding: utf_8 -*-
# @Create   : 2021/8/3 15:14
# @Author   : yh
# @Remark   : 读取配置文件的基类
from .exception import FileError


class ConfBase:

    def __init__(self):
        self._conf_path = None
        self._conf = None

    @property
    def conf_path(self):
        """
        配置文件的路径
        :return:
        """
        return self._conf_path

    @conf_path.setter
    def conf_path(self, value):
        import os
        self._conf_path = os.path.join(__file__.split('webexpress')[0], 'python', 'python_review', 'utils', 'conf',
                                       'module_conf', '%s.ini' % value)

    @property
    def conf(self):
        if self._conf:
            return self._conf
        elif self.conf_path:
            from configparser import ConfigParser
            conf = ConfigParser()
            try:
                conf.read(self.conf_path, encoding='gbk')
            except UnicodeDecodeError:
                try:
                    conf.read(self.conf_path, encoding='utf-8')
                except UnicodeDecodeError:
                    conf.read(self.conf_path, encoding='gb2312')
                except Exception:
                    raise FileError('读取文件失败')
        else:
            raise FileError('没有指定文件名')



# coding: utf-8

from setuptools import setup, find_packages

setup(
    name='mxsoftpy',
    version='0.0.18',
    author='great',
    author_email='great@mxsoft.com',
    url='https://gitee.com/yuanhao_1998/mxsoftpy.git',
    description=u'美信python框架',
    packages=find_packages(),
    install_requires=['pydantic', 'typing-extensions'],
    entry_points={}
)



# -*- coding: utf_8 -*-
# @Create   : 2021/9/10 17:24
# @Author   : yh
# @Remark   : 所有的数据库类型

VDT_EMPTY = 0x00
VDT_STR = 0x01
VDT_I32 = 0x03
VDT_I64 = 0x04
VDT_I8 = 0x05
VDT_DOUBLE = 0x06
VDT_FLOAT = 0x07
VDT_TIME = 0x09
VDT_UI8 = 0x10
VDT_UI64 = 0x13
VDT_TIMESTR = 0x30

type_map = {
    'str': VDT_STR,
    'float': VDT_FLOAT,
    'double': VDT_DOUBLE,
    'int': VDT_I32,
    'int8': VDT_I8,
    'int64': VDT_I64,
    'ui8': VDT_UI8,
    'ui64': VDT_UI64,
    'time': VDT_TIME,
    'timestr': VDT_TIMESTR,
    'empty': VDT_EMPTY,

    'ConstrainedIntValue': VDT_I32,
    'ConstrainedFloatValue': VDT_FLOAT,
    'ConstrainedStrValue': VDT_STR,
    'ConstrainedDecimalValue': VDT_TIME
    # 其他的类型，以后用到了再添加
}

type_c_python = {
    VDT_STR: 'str',
    VDT_FLOAT: 'float',
    VDT_DOUBLE: 'float',
    VDT_I8: 'int',
    VDT_I32: 'int',
    VDT_I64: 'int',
    VDT_UI8: 'int',
    VDT_UI64: 'int',
    VDT_TIME: 'int',
    VDT_TIMESTR: 'str',
    VDT_EMPTY: 'NoneType'
}

type_c_str = {
    VDT_EMPTY: 'VDT_EMPTY',
    VDT_STR: 'VDT_STR',
    VDT_I32: 'VDT_I32',
    VDT_I64: 'VDT_I64',
    VDT_I8: 'VDT_I8',
    VDT_DOUBLE: 'VDT_DOUBLE',
    VDT_FLOAT: 'VDT_FLOAT',
    VDT_TIME: 'VDT_TIME',
    VDT_UI8: 'VDT_UI8',
    VDT_UI64: 'VDT_UI64',
    VDT_TIMESTR: 'VDT_TIMESTR'
}



# -*- coding: utf_8 -*-
# @Create   : 2021/5/27 15:34
# @Author   : yh
# @Remark   : http请求响应相关
class HttpCode:
    st_100_continue = 100
    st_101_switch_protocols = 101
    st_200_ok = 200
    st_201_created = 201
    st_202_accepted = 202
    st_203_non_authoritative_information = 203
    st_204_no_content = 204
    st_205_reset_content = 205
    st_206_partial_content = 206
    st_300_multiple_choices = 300
    st_301_moved_permanently = 301
    st_302_found = 302
    st_303_see_other = 303
    st_304_not_modified = 304
    st_305_use_porxy = 305
    st_306_unused = 306
    st_307_temp_redirct = 307
    st_400_bad_request = 400
    st_401_unauthorized = 401
    st_403_forbidden = 403
    st_404_not_found = 404
    st_405_mothed_not_allowed = 405
    st_406_not_accepttable = 406
    st_407_proxy_authentication_required = 407
    st_408_request_timeout = 408
    st_409_conflict = 409
    st_410_gone = 410
    st_411_length_required = 411
    st_412_precondition_failed = 412
    st_413_request_entity_too_large = 413
    st_414_request_uri_too_long = 414
    st_415_unsupported_media_type = 415
    st_416_requested_range_not_satisfiable = 416
    st_417_expectation_failed = 417
    st_500_internal_server_error = 500
    st_501_not_implemented = 501
    st_502_bad_getway = 502
    st_503_service_unavailable = 503
    st_504_getway_timeout = 504
    st_505_http_version_not_supported = 505


class WeMr:
    WE_MR_BADREQUEST = 400
    WE_MR_BUFFERTOOSMALL = 10
    WE_MR_COMMUNICATION_ERROR = 3
    WE_MR_FAILED = 2
    WE_MR_NOTFOUNDPAGE = 404
    WE_MR_OK = 1
    WE_MR_UNKNOWNTYPE = 40


class HttpType:
    delete = 6
    get = 1
    head = 4
    options = 3
    post = 2
    put = 5
    trace = 7
    unknown = 0

    type_dict = {
        0: 'UNKNOWN',
        1: 'GET',
        2: 'POST',
        3: 'OPTIONS',
        4: 'HEAD',
        5: 'PUT',
        6: 'DELETE',
        7: 'TRACE'
    }



# -*- coding: utf_8 -*-
# @Create   : 2021/5/27 14:36
# @Author   : yh
# @Remark   : 脚手架基类

import typing as t


class BaseMx:

    def __init__(self, **options: t.Any) -> None:
        self.url_map = dict()  # 路由字典
        self.session_handler = None

    def add_url_rule(self, url: str, func: t.Callable, **options: t.Any) -> None:
        """
        向url_map中添加路由对应关系
        :param url: url路径
        :param func: url路径对应的方法
        """
        self.url_map[url] = func

    def route(self, url: str, **options: t.Any) -> t.Callable:
        """
        添加路由装饰器
        :param url: url路径
        """
        def decorator(func):
            self.add_url_rule(url, func)
            return func

        return decorator

    def add_resource(self, resource: t.Callable, url: str, **options: t.Any) -> None:
        """
        绑定CBV视图
        :param resource: 视图名
        :param url: 要绑定的url路径
        """
        def view(session):  # 接受此session参数，但是类方法中不需要使用，从session_handler中可获取到
            cls = resource(session)
            return cls.dispatch_request()
        view.__name__ = resource.__name__
        self.add_url_rule(url, view)
